from collections import deque
import re

import pyglet
from pyglet import font
from pyglet.window import key

from cocos.actions import *
from cocos.director import director
from cocos.layer import Layer

from sprites import RobotSprite


class StatusLayer(Layer):
    def __init__(self):
        super(StatusLayer, self).__init__()

        width, height = director.get_window_size()
        ft = font.load('Mono', 8)
        self.input = font.Text(ft, 'Input: [ ]', x=10, y=height-13)
        self.stack = font.Text(ft, 'Stack: [ ]', x=10, y=height-26)

    def draw(self):
        self.stack.draw()
        self.input.draw()


class BackgroundLayer(Layer):
    def __init__(self):
        super(BackgroundLayer, self).__init__()

        self.image = pyglet.resource.image('background.png')

    def draw(self):
        width, height = director.get_window_size()
        self.image.blit(0, 0)
   

class InputLayer(Layer):
    def __init__(self, robot):
        super(InputLayer, self).__init__()

        self.robot = robot
        self.robot.stack.clear()

        self.message = pyglet.text.Label(
                'Please enter input:',
                font_name='Mono',
                font_size=12,
                x=100,
                y=director.get_window_size()[1]/2+30,
                halign=font.Text.LEFT,
                valign=font.Text.CENTER,
                batch=self.batch)

        self.program = pyglet.text.Label(
                '',
                multiline=True,
                font_name='Mono',
                font_size=12,
                x=100,
                y=director.get_window_size()[1]/2-10,
                halign=font.Text.LEFT,
                valign=font.Text.TOP,
                batch=self.batch)

        self.program.width = director.get_window_size()[0]-200

    def on_key_release(self, key_pressed, mod):
        if key_pressed == key.RETURN:
            self.robot.input = deque(re.split('[\s\n\r]+', self.program.text.strip()))
            director.scene.layers = director.scene.layers[:-2]
            director.window.pop_handlers()
            self.robot.next()
        elif key_pressed == key.BACKSPACE:
            self.program.text = self.program.text[:-1]

    def on_text(self, text):
        if ((self.program.text.lstrip().count(' ') < self.robot.max_input-1) or
             self.program.text.lstrip().count(' ') == self.robot.max_input-1 and text != ' '):
            self.program.text = self.program.text + text.replace('\r', '\n').upper()


