import pyglet
from pyglet import font
from pyglet.window import key

from cocos.director import director
from cocos.layer import Layer, ColorLayer
from cocos.scene import Scene
from cocos.actions import *

from layers import StatusLayer, InputLayer, BackgroundLayer
from instructions import Instructions
from sprites import RobotSprite, EvilSprite, PowerSprite, TokenSprite
from misc import BACKGROUND


director.LEVELS = []


class Intro(Layer):
    def __init__(self):
        super(Intro, self).__init__()

        director.LEVELS = [Level1, Level2, Level3, Level4, Level5, Won]

        self.text = pyglet.text.Label("""Last night, Earth was invaded by robots from outerspace. It is your job, Johnny, to use your privileged intellect and stop them.

You decided to use your recently built robot to defeat the invaders. Since it's a progammable robot, it should be able to handle the task very well. All it has to do is collect 5 power generators, and we can reverse their polarity and use it to shut off the enemy robots.

The only problem is that you have only taught it a few instructions! Well, he'll have to learn as you go, because the invading robots are quickly taking care of Earth.

Press I to read the robot's manual that you have written, or any other key to start your mission. During the game, press ENTER to input a program, restart a level or move to the next one.
        """,
        multiline=True,
        font_size=13,
        x=60,
        y=director.get_window_size()[1]/2,
        halign=font.Text.LEFT,
        valign=font.Text.CENTER,
        batch=self.batch)

        self.text.width = director.get_window_size()[0]-120

    def on_key_release(self, key_pressed, mod):
        director.replace(Scene(BACKGROUND, BackgroundLayer(), director.LEVELS.pop(0)(), StatusLayer()))
        if key_pressed == key.I:
            director.push(Scene(BACKGROUND, Instructions(), StatusLayer()))


class Level(Layer):

    max_input = 5

    def __init__(self):
        super(Level, self).__init__()

        self.robot = RobotSprite("", x=96, y=96, max_input=self.max_input)
        self.add(self.robot)
        self.added_input = False

    def on_key_release(self, key_pressed, mod):
        if key_pressed == key.RETURN:
            if not self.added_input:
                self.added_input = True
                input = InputLayer(self.robot)
                director.scene.add(9, ColorLayer(0.0, 0.0, 0.0, 0.5))
                director.scene.add(10, input)
                director.window.push_handlers(input)
            elif self.robot.won == 1:
                director.replace(Scene(BACKGROUND, BackgroundLayer(), director.LEVELS.pop(0)(), StatusLayer()))
            elif not self.robot.input:
                director.replace(Scene(BACKGROUND, BackgroundLayer(), self.__class__(), StatusLayer()))


class Level1(Level):

    max_input = 5

    def on_enter(self):
        self.robot.update_status_layer()
        self.robot.next()

        width, height = director.get_window_size()

        power = PowerSprite(x=width-96, y=height-128)
        self.add(power)
        self.robot.generators.append(power)

        evil = EvilSprite(x=width-160, y=height-64)
        evil.do(Repeat(Move((0, -128), 2)))
        self.add(evil)
        self.robot.enemies.append(evil)

        token = TokenSprite("EXCH", x=width-96, y=96)
        self.add(token)
        self.robot.tokens.append(token)
        

class Level2(Level):

    max_input = 7

    def on_enter(self):
        self.robot.update_status_layer()
        self.robot.next()

        width, height = director.get_window_size()

        power = PowerSprite(x=width-96, y=height-256)
        self.add(power)
        self.robot.generators.append(power)

        evil1 = EvilSprite(x=width-352, y=96)
        evil1.do(Repeat(Move((0, 4*64), 3)))
        self.add(evil1)
        self.robot.enemies.append(evil1)

        evil3 = EvilSprite(x=width-352+64*2, y=96+4*64)
        evil3.do(Repeat(Move((0, -4*64), 3)))
        self.add(evil3)
        self.robot.enemies.append(evil3)

        evil2 = EvilSprite(x=96, y=height-256)
        self.add(evil2)
        self.robot.enemies.append(evil2)

        token1 = TokenSprite("ROLL", x=160, y=160)
        self.add(token1)
        self.robot.tokens.append(token1)
        
        token2 = TokenSprite("0 MOVE", x=224, y=height-64)
        self.add(token2)
        self.robot.tokens.append(token2)
        
        token3 = TokenSprite("EXCH SUB MOVE", x=width-160, y=height-64)
        self.add(token3)
        self.robot.tokens.append(token3)


class Level3(Level):

    max_input = 8

    def on_enter(self):
        self.robot.update_status_layer()
        self.robot.next()

        width, height = director.get_window_size()

        power = PowerSprite(x=7.5*64, y=1.5*64)
        self.add(power)
        self.robot.generators.append(power)

        evil1 = EvilSprite(x=2.5*64, y=2.5*64)
        evil1.do(Repeat(Move((0, -2*64), 2)))
        self.add(evil1)
        self.robot.enemies.append(evil1)

        evil2 = EvilSprite(x=1.5*64, y=5.5*64)
        self.add(evil2)
        self.robot.enemies.append(evil2)

        evil3 = EvilSprite(x=4.5*64, y=3.5*64)
        evil3.do(Repeat(Move((0, 2*64), 2)))
        self.add(evil3)
        self.robot.enemies.append(evil3)

        token1 = TokenSprite("DUP", x=1.5*64,y=2.5*64)
        self.add(token1)
        self.robot.tokens.append(token1)

        token2 = TokenSprite("-1 MUL 0", x=3.5*64, y=3.5*64)
        self.add(token2)
        self.robot.tokens.append(token2)


class Level4(Level):

    max_input = 8

    def on_enter(self):
        self.robot.update_status_layer()
        self.robot.next()

        width, height = director.get_window_size()

        power = PowerSprite(x=4.5*64, y=5.5*64)
        self.add(power)
        self.robot.generators.append(power)

        evil1 = EvilSprite(x=3.5*64, y=5.5*64)
        evil1.do(Repeat(Move((0, -3*64), 2)))
        self.add(evil1)
        self.robot.enemies.append(evil1)

        evil2 = EvilSprite(x=4.5*64, y=3.5*64)
        evil2.do(Repeat(Move((3*64, 0), 2)))
        self.add(evil2)
        self.robot.enemies.append(evil2)

        token1 = TokenSprite("POP", x=4.5*64, y=1.5*64)
        self.add(token1)
        self.robot.tokens.append(token1)

        token2 = TokenSprite("{ MOVE POP }", x=3.0*64, y=1.5*64)
        self.add(token2)
        self.robot.tokens.append(token2)


class Level5(Level):

    max_input = 5

    def on_enter(self):
        self.robot.update_status_layer()
        self.robot.next()

        width, height = director.get_window_size()

        power = PowerSprite(x=9.5*64, y=1.5*64)
        self.add(power)
        self.robot.generators.append(power)

        token1 = TokenSprite("0 LT { 0 -2 MOVE } IF", x=8.5*64, y=3.5*64)
        self.add(token1)
        self.robot.tokens.append(token1)

        token2 = TokenSprite("MOVE", x=5.5*64, y=4.5*64)
        self.add(token2)
        self.robot.tokens.append(token2)

        evil1 = EvilSprite(x=8.5*64, y=1.5*64)
        evil1.do(Repeat(Move((0, -1*64), 1)))
        self.add(evil1)
        self.robot.enemies.append(evil1)

        evil2 = EvilSprite(x=2.5*64, y=3.5*64)
        evil2.do(Repeat(Move((2*64, -1*64), 1)))
        self.add(evil2)
        self.robot.enemies.append(evil2)

    def on_key_release(self, key_pressed, mod):
        if key_pressed == key.RETURN:
            if not self.added_input:
                self.added_input = True
                input = InputLayer(self.robot)
                director.scene.add(9, ColorLayer(0.0, 0.0, 0.0, 0.5))
                director.scene.add(10, input)
                director.window.push_handlers(input)
            elif self.robot.won == 1:
                director.replace(Scene(BACKGROUND, director.LEVELS.pop(0)()))
            elif not self.robot.input:
                director.replace(Scene(BACKGROUND, BackgroundLayer(), self.__class__(), StatusLayer()))


class Won(Layer):
    def __init__(self):
        super(Won, self).__init__()

        self.text = pyglet.text.Label("""Congratulations, Johnny! You and your robot managed to collect all the power generators we needed in time. Our scientists have already reversed their polarities, and all over the world alien robots are being shut down.

Thanks to you, the world is again a safe place to live... at least until your next adventure!
        """,
        multiline=True,
        font_size=13,
        x=60,
        y=director.get_window_size()[1]/2,
        halign=font.Text.LEFT,
        valign=font.Text.CENTER,
        batch=self.batch)

        self.text.width = director.get_window_size()[0]-120

    def on_key_release(self, key_pressed, mod):
        director.pop()
